import { rootDir } from 'dante';
import { glob } from 'glob';
import { load, loadAll } from 'js-yaml';
import { existsSync } from 'node:fs';
import { readFile, readdir } from 'node:fs/promises';
import { resolve } from 'node:path';
import { pusherConfig } from '../configuration.js';
let commonCache;
let allTalksCache;
const themesCache = new Map();
const talksCache = new Map();
export async function resolvePusher() {
    let pusherFile = '';
    let pusher = '';
    if (pusherConfig) {
        const location = await glob(resolve(rootDir, 'node_modules/**/pusher-js/dist/web/pusher.js'), {
            follow: true,
            dot: true
        });
        if (!location.length) {
            throw new Error('Cannot find pusher-js module.');
        }
        pusherFile = location[0];
        pusher = await readFile(pusherFile, 'utf8');
    }
    return [
        pusherFile,
        pusher
    ];
}
export function resolveImagePath(theme, talk, url) {
    return resolve(rootDir, 'src', (url ?? '').replace('@talk', `talks/${talk}/assets`).replace('@theme', `themes/${theme}/assets`));
}
export function resolveImageUrl(theme, talk, url) {
    return (url ?? '').replace('@talk', `/assets/talks/${talk}`).replace('@theme', `/assets/themes/${theme}`);
}
export async function getCommon() {
    if (commonCache) {
        return commonCache;
    }
    let loaded = {};
    const commonPath = resolve(rootDir, 'src/talks', 'common.yml');
    if (existsSync(commonPath)) {
        loaded = await load(await readFile(commonPath, 'utf8'));
    }
    commonCache = loaded;
    return loaded;
}
export async function getTheme(themeName) {
    const cached = themesCache.get(themeName);
    if (cached) {
        return cached;
    }
    let fontsStyles = '';
    const fontsUrls = [];
    const themeFile = await readFile(resolve(rootDir, 'src/themes', themeName, 'theme.yml'), 'utf8');
    const rawTheme = load(themeFile);
    const fonts = rawTheme.fonts;
    // Generate all the fonts related CSS
    if (fonts.families && fonts.ranges) {
        for (const [name, styles] of Object.entries(fonts.families)){
            for (const [style, weights] of Object.entries(styles)){
                for (const [weight, definition] of Object.entries(weights)){
                    for (const [range, url] of Object.entries(definition)){
                        fontsStyles += `
@font-face {
  font-family: "${name}";
  font-style: ${style};
  font-weight: ${weight};
  font-display: swap;
  src: url(${url}) format('woff2');
  unicode-range: ${fonts.ranges[range]}
}
        `.trim() + '\n';
                        fontsUrls.push(url);
                    }
                }
            }
        }
    }
    const theme = {
        id: themeName,
        urls: {},
        ...rawTheme,
        images: (rawTheme.images ?? []).map((i)=>resolveImageUrl(themeName, '', i)),
        fontsStyles,
        fontsUrls
    };
    themesCache.set(themeName, theme);
    return theme;
}
export async function getTalk(id) {
    const cached = talksCache.get(id);
    if (cached) {
        return cached;
    }
    let talk;
    if (existsSync(resolve(rootDir, 'src/talks', id, 'slides.yml'))) {
        const infoFile = await readFile(resolve(rootDir, 'src/talks', id, 'info.yml'), 'utf8');
        talk = load(infoFile);
        const slidesFile = await readFile(resolve(rootDir, 'src/talks', id, 'slides.yml'), 'utf8');
        talk.slides = loadAll(slidesFile);
    } else {
        const talkFile = await readFile(resolve(rootDir, 'src/talks', id, 'talk.yml'), 'utf8');
        talk = load(talkFile);
    }
    const common = await getCommon();
    if (typeof talk.config === 'string' && talk.config === 'common.config') {
        talk.config = common.config;
    }
    for (const [key, value] of Object.entries(talk.document)){
        if (typeof value === 'string' && value.startsWith('common.')) {
            talk.document[key] = common[value.replace('common.', '')];
        }
    }
    // Gather all images
    const images = [];
    for (const slide of talk.slides){
        // Track the included images
        for (const property of [
            'image'
        ]){
            const image = slide[property];
            if (image) {
                images.push(resolveImageUrl(talk.config.theme, id, image));
            }
        }
    }
    // Set some properties
    talk.id = id;
    talk.config.urls = {
        ...talk.config.urls
    };
    talk.slidesCount = talk.slides.length;
    talk.slidesPadding = Math.ceil(Math.log10(talk.slides.length));
    talk.aspectRatio = talk.config.dimensions.width / talk.config.dimensions.height;
    talk.images = images;
    talksCache.set(id, talk);
    return talk;
}
export async function getAllTalks() {
    if (allTalksCache) {
        return allTalksCache;
    }
    const allFiles = await readdir(resolve(rootDir, 'src/talks'));
    allTalksCache = new Set(allFiles.filter((talk)=>{
        return existsSync(resolve(rootDir, 'src/talks', talk, 'talk.yml')) || existsSync(resolve(rootDir, 'src/talks', talk, 'slides.yml'));
    }));
    return allTalksCache;
}
